@@ -18,8 +18,10 @@
 /**
  * Name........: princeprocessor (pp)
  * Description.: Standalone password candidate generator using the PRINCE algorithm
- * Version.....: 0.21
- * Autor.......: Jens Steube <jens.steube@gmail.com>
+ * Version.....: 0.22
+ * Authors.....: Jens Steube <jens.steube@gmail.com>
+ *               Steve Thomas (Sc00bz)
+ *               magnum <john.magnum@hushmail.com>
  * License.....: MIT
  */
 
@@ -37,7 +39,7 @@
 #define SAVE_POS      1
 #define SAVE_FILE     "pp.save"
 
-#define VERSION_BIN   21
+#define VERSION_BIN   22
 
 #define ALLOC_NEW_ELEMS  0x40000
 #define ALLOC_NEW_CHAINS 0x10
@@ -195,7 +197,7 @@ static const char *USAGE_BIG[] =
   "       --elem-cnt-max=NUM    Maximum number of elements per chain",
   "       --wl-dist-len         Calculate output length distribution from wordlist",
   "       --wl-max=NUM          Load only NUM words from input wordlist or use 0 to disable",
-  "  -c,  --dupe-check-disable  Disable dupes check for faster inital load",
+  "  -c,  --dupe-check-disable  Disable dupes check for faster initial load",
   "       --save-pos-disable    Save the position for later resume with -s",
   "",
   "* Resources:",
@@ -660,7 +662,7 @@ static void add_uniq (db_entry_t *db_entry, char *input_buf, int input_len)
 
 mpz_t save;
 
-static void catch_int ()
+static void catch_int (int signum)
 {
   FILE *fp = fopen (SAVE_FILE, "w");
 
@@ -672,7 +674,7 @@ static void catch_int ()
 
   fclose (fp);
 
-  exit (0);
+  exit (signum==0?0:signum);
 }
 
 int main (int argc, char *argv[])
@@ -761,8 +763,8 @@ int main (int argc, char *argv[])
       case IDX_CASE_PERMUTE:          case_permute      = 1;              break;
       case IDX_DUPE_CHECK_DISABLE:    dupe_check        = 0;              break;
       case IDX_SAVE_POS_DISABLE:      save_pos          = 0;              break;
-      case IDX_SKIP:                  mpz_set_str (skip,  optarg, 0);     break;
-      case IDX_LIMIT:                 mpz_set_str (limit, optarg, 0);     break;
+      case IDX_SKIP:                  mpz_set_str (skip,  optarg, 10);    break;
+      case IDX_LIMIT:                 mpz_set_str (limit, optarg, 10);    break;
       case IDX_OUTPUT_FILE:           output_file       = optarg;         break;
 
       default: return (-1);
@@ -823,9 +825,9 @@ int main (int argc, char *argv[])
     return (-1);
   }
 
-  if (elem_cnt_max <= 0)
+  if (elem_cnt_max < (1 - pw_max))
   {
-    fprintf (stderr, "Value of --elem-cnt-max (%d) must be greater than %d\n", elem_cnt_max, 0);
+    fprintf (stderr, "Value of --elem-cnt-max (%d) must be greater than %d\n", elem_cnt_max, (0 - pw_max));
 
     return (-1);
   }
@@ -837,7 +839,7 @@ int main (int argc, char *argv[])
     return (-1);
   }
 
-  if (elem_cnt_min > elem_cnt_max)
+  if (elem_cnt_max > 0 && elem_cnt_min > elem_cnt_max)
   {
     fprintf (stderr, "Value of --elem-cnt-min (%d) must be smaller or equal than value of --elem-cnt-max (%d)\n", elem_cnt_min, elem_cnt_max);
 
@@ -1078,7 +1080,20 @@ int main (int argc, char *argv[])
 
       if (valid2 == 0) continue;
 
-      int valid3 = chain_valid_with_cnt_max (&chain_buf_new, elem_cnt_max);
+      int eff_elem_cnt_max;
+
+      if (elem_cnt_max > 0)
+      {
+        eff_elem_cnt_max = elem_cnt_max;
+      }
+      else
+      {
+        eff_elem_cnt_max = pw_len + elem_cnt_max;
+
+        if (eff_elem_cnt_max <= elem_cnt_min) continue;
+      }
+
+      int valid3 = chain_valid_with_cnt_max (&chain_buf_new, eff_elem_cnt_max);
 
       if (valid3 == 0) continue;
 
@@ -1475,7 +1490,7 @@ int main (int argc, char *argv[])
 
   if (save_pos)
   {
-    catch_int ();
+    catch_int (0);
   }
 
   /**
